/* $ Jonathan Katz: incsim.g,v 1.3 1997/03/10 23:26:36 jkatz Exp $
** Adapted by Jason Wittenberg 12/20/2001
** witty@polisci.wisc.edu
** 
** Purpose: Calculate Incumbency effects via simulation.
*/

proc (6) = IncSim(theta,vc,dta,iv,ivc,M);
  local bk,qv1,qv2,qv3,pcpfc,pcppc,dstemp,NC,temp1,temp2;
  clearg msecon,mselab,mselib,mse,msefc,msepc,qpca,qttwev1,qttwev2,qttwev3;
  clearg k,N,P,ThetaM,X1,X2,k1,iv1,eiv1,XO1,eivc1,XI1,k2,iv2,eiv2,XO2,eivc2,
        XI2,tmarker,j,l,thetaj,b1j,b2j,dfj,Sigmaj,YOj,VOj,YIj,VIj,effj,Eff,Ej,
    ivc1,ivc2,ME,ce,Ye,PV_e,EV_e,tempev,tempy,
    dta1,dta2,v1,v2,v3,Yj,Pvj,ttwev,ttwpv,yhat,tpv1,tpv2,tpv3,sigma,dof,MP,simelec,
      ttwpv1,ttwpv2,ttwpv3,ttwev1,ttwev2,ttwev3,pcp,fullx1,fullx2,pcptemp,pcpci,
      mttwev1,mttwev2,mttwev3;

  Eff = {};
  Pvj = {};
  ttwpv1 = {};                  /* These hold the predicted and expected values of V */
  ttwpv2 = {};
  ttwpv3 = {};
  ttwev1 = {};   
  ttwev2 = {};
  ttwev3 = {};
  
  ttwev = {};
  pcp = {};
  simelec = {};
  

     
  /*** Useful Values ***/
  k = rows(theta);
  N = rows(lget(dta,1));
  P = 2;
  ME = 300;                     /* This is the number of terms drawn from MVT */
  NC=0.001;
  
  
  /*** Draw the M paramaters from multivariate T ***/
  ThetaM = theta + rndmn(zeros(k,1),vc,M)';   /* 16x500 */
  
  tempy=2+exp(ThetaM[13,.]);
  dfpost = dfpost ~ tempy';     /* Note: dfpost initialized in MAIN.PRG */
  
  
  
  /*** Seperate Out X1 and X2 ***/
  X1 = lget(dta,1);
  X1 = X1[.,2:cols(X1)];                     /* Original Data for 1, Nx6 */
  FULLX1=X1;
  k1 = cols(X1);
  
  
  /*  This does "setx" for party 1 */
  X1 = meanc(X1)';
  
  iv1 = lget(iv,1)';
  XO1 = substute(X1,iv1,0);         /* Data for 1 assuming Open Seat */
  ivc1 = lget(ivc,1)';
  XI1 = substute(XO1,ivc1,1);       /* Data for 1 assuming Inc Seat  */

  
  X2 = lget(dta,2);
  X2 = X2[.,2:cols(x2)];             /* Original Data for 2            */
  FULLX2=X2;
  k2 = cols(X2);
  
  
  /*  This does "setx" for party 2 */
  X2 = meanc(X2)';
  
  iv2 = lget(iv,2)';
  XO2 = substute(X2,iv2,0);         /* Data for 2 assuming Open Seat */
  ivc2 = lget(ivc,2)';
  XI2 = substute(XO2,ivc2,1);       /* Data for 2 assuming Inc Seat  */
 

  /*** Create Marker to pull elements of theta ***/ 
  tmarker = 1*ones(k1,1) | 2*ones(k2,1) | 3 | 4*ones(P*(P+1)/2,1);
  
  Yhat=FULLX1*theta[1:k1]~FULLX2*theta[k1+1:k1+k2];      
  
  /*** Loop for Paramater draws ***/
  for j (1,M,1);
    /* Grab appropriate theta from draws */ 
    thetaj = ThetaM[.,j];
    
    /* Transform the Paramaters */
    b1j = selif(thetaj,tmarker.==1);       /* 6x1, for IVs */
    b2j = selif(thetaj,tmarker.==2);
    dfj = tdfi(p,thetaj);                  /* Convert back to real scale */
    Sigmaj = tsigi(p,thetaj);
    
    /* Draw Errors */
    Ej = rndmt(zeros(p,1),Sigmaj,dfj,ME);  /* 300x2 of these */
    
/* Do KK Incumbency effect */    

     /* Open Seat Predictions */
    YOj =  (XO1*b1j ~ XO2*b2j) + Ej;
    VOj = meanc(mnli(YOj))';
    
    /* Inc Seat Predictions */
    YIj = (XI1*b1j ~ XI2*b2j) + Ej;
    VIj = meanc(mnli(YIj))';

    effj = VIj-VOj;
    Eff = Eff | effj;
    
/* End of KK part */
     
    /* Draw Errors */
    Ej = rndmt(zeros(p,1),Sigmaj,dfj,N);   /* Nx2 */
    Ej = substute(Ej,Ej.>700,700);         /* Cap Ej at 700 to avoid missing values in Pvj */
    Ej = substute(Ej,Ej.<-700,-700);
    Ej = missrv(Ej,0);
       
    Yj=(FULLX1* b1j ~ FULLX2*b2j) + Ej;
    
    Pvj=mnli(Yj);                          /* Nx3 */
 
  /* At this point ttw vars hold predicted values,
     but see below */
    ttwpv1=ttwpv1 ~ Pvj[.,1];              /* Will be NxMSIMS by the end of FOR loop */
    ttwpv2=ttwpv2 ~ Pvj[.,2];
    ttwpv3=ttwpv3 ~ Pvj[.,3];
    
    /* Draw Errors for Computing EV's */
    
    Ej = rndmt(zeros(p,1),Sigmaj,dfj,ME);
    Ej = substute(Ej,Ej.>700,700);         /* Cap Ej at 700 to avoid missing values in Pvj */
    Ej = substute(Ej,Ej.<-700,-700);
    Ej = missrv(Ej,0);
    tempev = {};
    for ce (1,N,1);                        /* N is number of observations */
      
      Ye=(FULLX1[ce,.]* b1j ~ FULLX2[ce,.]*b2j) + Ej;   /* MEx2 */
      PV_e = mnli(Ye);
      EV_e = meanc(PV_e);
      
      tempy=ismiss(PV_e);
      if tempy==1;
        "Found a missing values in PV_e!";
        "Obsevation Iteration Number= " ce;
        "Stopping program - check Ej in INCSIM.G";
        stop;
      endif;
    
      tempev = tempev ~ EV_e;
            
    endfor;
  
    tempev = tempev';             /* Take transpose to yield Nx3 matrix */
    simelec = simelec | tempev;   /* Take transpose to yield Nx3 matrix - one simulated election */
    
    ttwev1=ttwev1 ~ tempev[.,1];  /* Eventually these will be NxMSIMS posteriors */  
    ttwev2=ttwev2 ~ tempev[.,2];
    ttwev3=ttwev3 ~ tempev[.,3];
    
  endfor;
  
  mttwev1=meanc(ttwev1');         /* Mean of the posterior---> yields ev1 */
  mttwev2=meanc(ttwev2');
  mttwev3=meanc(ttwev3');

  ttwev = mttwev1~mttwev2~mttwev3;   /* This gets passed back to main program */
 
      
  clear tempy;  
     
  /* Compute % correctly predicted */
  
  /** Grab Vote Shares **/
  dta1 = lget(dta,1);
  v1 = dta1[.,1];
  
  dta2 = lget(dta,2);
  v2 = dta2[.,1];

  v3 = 1-v1-v2;
  
  /* The following code is adapted from COMP2.SRC. Need to compute PCP
     separately for fully and partially contested districts */
  
  MP = -9*ones(rows(v1),1);
  MP = substute(MP,v1.>NC .and v2.>NC .and v3.>NC,0);
  MP = substute(MP,v1.<=NC .and v2.>NC .and v3.>NC,1);
  MP = substute(MP,v1.>NC .and v2.<=NC .and v3.>NC,2);
  MP = substute(MP,v1.>NC .and v2.>NC .and v3.<=NC,3);
  MP = substute(MP,v1.>NC .and v2.<=NC .and v3.<=NC,4);
  MP = substute(MP,v1.<=NC .and v2.>NC .and v3.<=NC,5);
  MP = substute(MP,v1.<=NC .and v2.<=NC .and v3.>NC,6);
  
  /* Compute for fully contested */
  
  qttwev1=selif(ttwev1,MP.==0);
  qttwev2=selif(ttwev2,MP.==0);
  qttwev3=selif(ttwev3,MP.==0);
 
  qv1=selif(v1,MP.==0);
  qv2=selif(v2,MP.==0);
  qv3=selif(v3,MP.==0);
 
    
  for ce (1,M,1);
    tempy=maxindc((qttwev1[.,ce]~qttwev2[.,ce]~qttwev3[.,ce])');
    pcp=pcp~tempy;
  endfor;
  pcptemp=pcp;    /* N x MSIMS--each the winner of the simulated election */
  
 
   
  tempy=maxindc((qv1~qv2~qv3)');   /* N x 1 */
  
  pcp = (tempy.==pcptemp);    /* NxMSIMS vector of 1's and 0's vector. 
                                 1's=correctly predicted */
  
  pcp = sumc(pcp)./rows(qv1); /* Sum up 1's and divide by N to get fractions 
                                 This is the posterior of PCP-- SIMSx1*/
                         
  { pcpci } = makeci(pcp,95);
  
  pcp=meanc(pcp);             /* Average PCP */
  
  pcpfc = pcpci[1,1]~pcp~pcpci[1,2];
  clearg pcp;
  
  /* Compute for partially contested */
  
  qttwev1=selif(ttwev1,(MP./=0 .and MP./=-9));
  qttwev2=selif(ttwev2,(MP./=0 .and MP./=-9));
  qttwev3=selif(ttwev3,(MP./=0 .and MP./=-9));
 
  qv1=selif(v1,(MP./=0 .and MP./=-9));
  qv2=selif(v2,(MP./=0 .and MP./=-9));
  qv3=selif(v3,(MP./=0 .and MP./=-9));
  
  if ismiss(qttwev1);
     pcppc=-1~-1~-1;
  else;
   pcp = {};       
   for ce (1,M,1);
     tempy=maxindc((qttwev1[.,ce]~qttwev2[.,ce]~qttwev3[.,ce])');
     pcp=pcp~tempy;
   endfor;
   pcptemp=pcp;              /* NxMSIMS */
   
   tempy=maxindc((qv1~qv2~qv3)');   /* N x 1 */
  
   pcp = (tempy.==pcptemp);         /* 1's and 0's vector. 1's=correctly predicted */
   pcp = sumc(pcp)./rows(qv1);      /* Sum up 1's and divide by N to get fractions 
                         This is the posterior of PCP-- MSIMSx1*/
                         
   { pcpci } = makeci(pcp,95);
  
   pcp=meanc(pcp);                  /* Average PCP */
  
   pcppc = pcpci[1,1]~pcp~pcpci[1,2];

  endif;
  pcp=pcpfc~pcppc;                  /* Concatenate them together to pass to main program */
  
  /* Compute MSE */
  
  /* Fully contested */
  
  qpca = lget(dta,3);
  
  qttwev1=100.*selif(ttwev1,MP.==0);
  qttwev2=100.*selif(ttwev2,MP.==0);
  qttwev3=100.*selif(ttwev3,MP.==0);
  
  qpca=selif(qpca,MP.==0);
 
  qv1=100.*selif(v1,MP.==0);
  qv2=100.*selif(v2,MP.==0);
  qv3=100.*selif(v3,MP.==0);
  
    
  msecon=meanc(((qv1-qttwev1)^2+stdc(qttwev1')^2)');
  mselab=meanc(((qv2-qttwev2)^2+stdc(qttwev2')^2)');
  mselib=meanc(((qv3-qttwev3)^2+stdc(qttwev3')^2)');
  msefc=qpca~msecon~mselab~mselib;
  
  /* Partially contested */
  
  qpca = lget(dta,3);
  
  qttwev1=100.*selif(ttwev1,(MP./=0 .and MP./=-9));
  qttwev2=100.*selif(ttwev2,(MP./=0 .and MP./=-9));
  qttwev3=100.*selif(ttwev3,(MP./=0 .and MP./=-9));
  
  qpca=selif(qpca,(MP./=0 .and MP./=-9));
 
  qv1=100.*selif(v1,(MP./=0 .and MP./=-9));
  qv2=100.*selif(v2,(MP./=0 .and MP./=-9));
  qv3=100.*selif(v3,(MP./=0 .and MP./=-9));
  
  
  if ismiss(qttwev1);
     msepc={};
  else;
    temp1=100.*(qttwev1./(qttwev1+qttwev2));
    temp2=100.*(qttwev2./(qttwev1+qttwev2));
    msecon=meanc(((qv1-temp1)^2+stdc(temp1')^2)');
    mselab=meanc(((qv2-temp2)^2+stdc(temp2')^2)');
    msepc=qpca~msecon~mselab~(-1.*ones(rows(qttwev1),1)); 
  endif;
  
  mse=msefc | msepc;
  mse=sortc(mse,1);
  mse=mse[.,2:4];    /* Get rid of pca */
 
  /* Note that the NxMSIMS matrices should
     be transposed to MSIMSxN, so that each
     column contains simulated values that
     can be averaged, etc. */
     
     tpv1=ttwpv1';
     tpv2=ttwpv2';
     tpv3=ttwpv3';
        
  /* Take the mean of the predicteds */
  
  ttwpv1=meanc(tpv1);  
  ttwpv2=meanc(tpv2);
  ttwpv3=meanc(tpv3);
  
  ttwpv = ttwpv1~ttwpv2~ttwpv3;
    
   retp(Eff,ttwpv,ttwev,yhat,pcp,mse);
endp;
